<?php

/*
 * NOTICE OF LICENSE
 *
 * This source file is subject to a commercial license from SILADEL SAS.
 * Use, copy, modification or distribution of this source file without written
 * accord from by S.A.S. SILADEL is strictly forbidden.
 * In order to obtain a license, please contact us: info@siladel.fr
 */

/*
 * LICENCE D'UTILISATION
 * 
 * L'utilisation de ce fichier source est soumise a une licence commerciale
 * concedee par la societe SILADEL SAS.
 * Toute utilisation, reproduction, modification ou distribution du present
 * fichier source sans contrat de licence délivré par la societe SILADEL SAS. est strictement interdite.
 * Pour obtenir une licence, veuillez nous contacter à l'adresse : info@siladel.fr
 */

/**
 * @package     EANPRODOC
 * @author      David I.
 * @copyright   Copyright 2020, SILADEL SAS 
 */

if (file_exists(dirname(__FILE__) . '/eanprodoc.class.php')) {
    require_once dirname(__FILE__) . '/eanprodoc.class.php';
}
require_once DOL_DOCUMENT_ROOT . '/core/class/html.form.class.php';

require_once DOL_DOCUMENT_ROOT . '/product/class/product.class.php';


class ActionsEanprodoc {

    /**
     * @var type array database
     */
    protected $db;

    /**
     * @var bool permission
     */
    public $permit;
  
    /**
     * @var array error table
     */
    public $errors = array();
    
    /**
     * @var objet form
     */
    public $form;

    /*
     * @var objet product
     */
    public $product;

    /**
     * 	Constructor
     * 	@param	DoliDB	$db	Database handler
     */
    function __construct(DoliDB $db) {
        $this->db = $db;
        
        $this->form = new Form($this->db);
        $this->product = new Product($this->db);
    }
    
    /**
     * Overload of pdf_writelinedesc (description of the product lines)
     * @global object $conf
     * @global type $langs
     * @param type $parameters      Hook metadatas (context, etc...)
     * @param type $object          The object to process (an invoice if you are in invoice module, a propale in propale's module, etc...)
     * @param type $action          Current action (if set). Generally create or edit or null
     * @return int  < 0 on error, 0 on success
     */
     
    function pdf_writelinedesc(&$parameters, &$object, &$action) {
        global $conf, $langs;
     
        $error = 0; // Error counter
        
        $decal_droite = 30; // decalage
        $decal_gauche = 30; // decalage
        
        $langs->load("eanprodoc@eanprodoc");
        
        $modelPdf = $object->modelpdf;
        
        if(!empty($conf->global->EANPRODOC_EXCLU)) {
            $tplExclu = $conf->global->EANPRODOC_EXCLU;
            
             if (preg_match('/'.$tplExclu.'/i', $modelPdf)) {
                 return 0;
             }
        }

        if (in_array('expeditioncard', explode(':', $parameters['context'])) && $conf->global->EANPRODOC_EXP
                || in_array('invoicecard', explode(':', $parameters['context'])) && $conf->global->EANPRODOC_FACT
                || in_array('ordercard', explode(':', $parameters['context'])) && $conf->global->EANPRODOC_COM
                || in_array('ordersuppliercard', explode(':', $parameters['context'])) && $conf->global->EANPRODOC_SCOM
                || in_array('invoicesuppliercard', explode(':', $parameters['context'])) && $conf->global->EANPRODOC_SFACT
                || in_array('bonderetourcard', explode(':', $parameters['context'])) && $conf->global->EANPRODOC_BR)
        {
       
            $fk_product = $object->lines[$parameters['i']]->fk_product;

            if ((1 == $this->product->fetch($fk_product)) && !empty($this->product->barcode)) {
                    $barcode_value = $this->product->barcode;

                    $barcode_id = $this->product->barcode_type;

                    $barcode_type = $this->getBarCodeType($barcode_id);

                    $object->lines[$parameters['i']]->barcode = $barcode_value;
                    $object->lines[$parameters['i']]->barcode_type = $barcode_id;
            } else {
                    return 0;
            }

            $this->form->showbarcode($object);

            $style = array( 
                    'position' => '',
                    'align' => 'T',
                    'stretch' => false,
                    'fitwidth' => false,
                    'cellfitalign' => 'T',
                    'border' => false,
                    'hpadding' => '1',
                    'vpadding' => '1',
                    'fgcolor' => array(0,0,0),
                    'bgcolor' => false, //array(0,155,255),
                    'text' => ($conf->global->EANPRODOC_PCODE),
                    'font' => 'helvetica',
                    'fontsize' => 8,
                    'stretchtext' => 3
            );

            if(($barcode_type == "DATAMATRIX") || ($barcode_type == "QRCODE")) {
                    return 0;
            }
            
            if($barcode_type == "C128") {
                    $decal_droite = 50; // decalage
                    $decal_gauche = 50; // decalage
            }

            // test du code barre
            if($barcode_type == "EAN13") {
                    $lenbarcode = strlen($barcode_value);
                    $barcode_test = $barcode_value;
                    if($lenbarcode == 13 && !$this->TestCheckDigit($barcode_test)) {
                            $code_msg = substr($barcode_test, -13, 12) . '<b style="color: red">' .substr($barcode_test, -1). '</b>';

                            setEventMessages($langs->trans("eanprodoc_errorbarrecode", $code_msg, $this->GetCheckDigit($barcode_test)), null, 'warnings');
                            return 0;
                    }

                    if ($lenbarcode > 13) {
                            setEventMessages($langs->trans("bad barcode"), null, 'warnings');
                            return 0;
                    }
            } else if ($barcode_type != "C128"){
                setEventMessages($langs->trans("bad barcode type") . ' ' . $barcode_type, null, 'warnings');
                return 0;
            }

            if (!empty($barcode_value) && strlen($barcode_value) > 7) {
                    $labelproductservice = pdf_getlinedesc($object, $parameters['i'], $parameters['outputlangs'], $parameters['hideref'], $parameters['hidedesc'], $parameters['issupplierline']);
                    $labelproductservice = preg_replace('/(<img[^>]*src=")([^"]*)(&amp;)([^"]*")/', '\1\2&\4', $labelproductservice, -1, $nbrep);
                    $parameters['pdf']->writeHTMLCell($parameters['w'] - $decal_gauche, $parameters['h'], $parameters['posx'] + $decal_droite, $parameters['posy'], $parameters['outputlangs']->convToOutputCharset($labelproductservice), 0, 1, false, true, 'J', true);
                    $y = $parameters['pdf']->GetY();
                    $parameters['pdf']->write1DBarcode($barcode_value, $barcode_type, $parameters['posx'], $y - 5, $decal_droite, 10, 0.4, $style, 'B');

            } else {
                    setEventMessages($langs->trans("bad barcode") . ' ' . $barcode_value, null, 'warnings');
                    return 0;
            }

            if (! $error) {
                    //$this->results = array('myreturn' => $barcode_value);
                    //$this->resprints = (string)$barcode_value;
                    return 1; // or return 1 to replace standard code
            } else {
                    $this->errors[] = 'Error message';

                    return -1;
            }
        }
        return 0;
    }
    
    
    /**
     * get barcode type
     * @param int $id rowid
     * @return string type de code => EAN13 etc..
     */
    public function getBarCodeType($id) {
        $sql = 'SELECT `rowid`, `code`';
        $sql .= ' FROM ' . MAIN_DB_PREFIX . 'c_barcode_type';
        $sql .= ' WHERE `rowid` = ' .(int)$id.  '';
        
        $resql = $this->db->query($sql);
        
        if ($resql) {
            $objp = $this->db->fetch_object($resql);
			
            if($objp->code)
                $this->db->free($result);
                return $objp->code;
        }

        return 'EAN13';
    }
     
    /**
     * retour la clé de ocntrôle
     * @param type $barcode
     * @return type
     */
    function GetCheckDigit($barcode) {
        //Compute the check digit
        $sum=0;
        for($i=1;$i<=11;$i+=2)
            $sum+=3*$barcode[$i];
        for($i=0;$i<=10;$i+=2)
            $sum+=$barcode[$i];
        $r=$sum%10;
        if($r>0)
            $r=10-$r;
        return $r;
    }
    
    /**
     * Test la clé de contrôle
     * @param type $barcode
     * @return type
     */
    function TestCheckDigit($barcode) {
        //Test validity of check digit
        $sum=0;
        for($i=1;$i<=11;$i+=2)
            $sum+=3*$barcode[$i];
        for($i=0;$i<=10;$i+=2)
            $sum+=$barcode[$i];
        return ($sum+$barcode[12])%10==0;
    }
}
